@@ -27,7 +27,7 @@ arg(atversion_ARG, '\0', "atversion", string_arg, 0)
 arg(binary_ARG, '\0', "binary", NULL, 0)
 arg(bootloaderareasize_ARG, '\0', "bootloaderareasize", size_mb_arg, 0)
 arg(cache_long_ARG, '\0', "cache", NULL, 0)
-arg(cachemode_ARG, '\0', "cachemode", string_arg, 0)
+arg(cachemode_ARG, '\0', "cachemode", cachemode_arg, 0)
 arg(cachepool_ARG, '\0', "cachepool", string_arg, 0)
 arg(commandprofile_ARG, '\0', "commandprofile", string_arg, 0)
 arg(config_ARG, '\0', "config", string_arg, 0)
