@@ -217,7 +217,7 @@ int lvrename(struct cmd_context *cmd, int argc, char **argv)
 
 	handle->custom_handle = &lp;
 
-	ret = process_each_vg(cmd, 0, NULL, vg_name, READ_FOR_UPDATE, handle,
+	ret = process_each_vg(cmd, 0, NULL, vg_name, NULL, READ_FOR_UPDATE, handle,
 			      _lvrename_single);
 
 	destroy_processing_handle(cmd, handle);
